
local frame = CreateFrame("frame", null, UIParent);
frame:RegisterEvent("PLAYER_TARGET_CHANGED");
frame:SetScript("OnEvent", function()

        local name = UnitName("target")

        if name == nil then
            return
        end

        if Gankr == nil then
            return
        end

        local ganker = GetGankerByName(name)

        if ganker ~= nil and ganker.active == 1 then
            local reason = Gankr[ganker.index].reason

            if reason == nil then
                reason = ""
            end

            print("Ganker detected: " .. name .. "! Finish him!" .. " Reason: " .. reason)
            PlaySound(SOUNDKIT.RAID_WARNING)
        end

end);

local function NameExists(name)
    for i=1,#Gankr do
        if Gankr[i].name == name or string.lower(Gankr[i].name) == string.lower(name) then
            return true
        end
    end

    return false
end

local function SetActive(name)

    local ganker = GetGankerByName(name)

    if ganker == nil then
        return
    end

    ganker.active = 1
    SaveGanker(ganker)
end


function GetGankerByName(name)
    for i=1,#Gankr do
        if Gankr[i].name == name or string.lower(Gankr[i].name) == string.lower(name) then
            return Gankr[i]
        end
    end
    return nil
end


local function IncrementIndex()
    local index = Gankr["index"]
    index = index + 1
    Gankr["index"] = index
    return index
end


local function AddNameToGanklist(name, reason)
    local ganker = GetGankerByName(name)

    if ganker ~= nil then
        print("Name already exists! Changing to active if inactive.")
        SetActive(name)
        return
    end

    local newIndex = IncrementIndex()
    Gankr[newIndex] = {name = name, active = 1, reason = reason, index = newIndex}

    if reason ~= nil and reason ~= "" then
        print(name .. " added to the gank table. Reason: " .. reason)
    else
        print(name .. " added to the gank table.")
    end
end


function SaveGanker(ganker)
    Gankr[ganker.index] = ganker;
end


local function RemoveNameFromGankTable(name)

    if name == nil or name == "" then
        return
    end

    local ganker = GetGankerByName(name)

    if ganker ~= nil then
        ganker.active = 0
        SaveGanker(ganker)
        print(name .. " removed.")
    end
end

local function ListGankers()

    for i=1,#Gankr do
        if Gankr[i].active == 1 then

            local reason = Gankr[i].reason

            if reason == nil then
                reason = ""
            end

            print(Gankr[i].name .. " - " .. reason)

        end
    end
end


local function CreateExportText()

    local exportString = ""

    for i=1,#Gankr do
        local ganker = Gankr[i]
        exportString = exportString .. ganker.name .. "," .. ganker.reason .. "\n"
    end

    return exportString
end


function mysplit(s, delimiter)
    result = {};
    for match in (s..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match);
    end
    return result;
end

local function Import(importText)

    local allLines = mysplit(importText, "\n")

    for k, v in pairs(allLines) do

        local fields = mysplit(v, ",")


        if fields[1] == nil or fields[1] == "" then
        -- skip
        else
            AddNameToGanklist(fields[1], fields[2])

        end
    end
end

local function CreateImportFrame()

    local s = CreateFrame("ScrollFrame", nil, UIParent, "UIPanelScrollFrameTemplate") -- or you actual parent instead
    s:SetSize(300,200)
    s:SetPoint("CENTER")
    local e = CreateFrame("EditBox", nil, s)
    e:SetMultiLine(true)
    e:SetFontObject(ChatFontNormal)
    e:SetWidth(300)
    s:SetScrollChild(e)
    --- demo multi line text
    e:SetText("")
    s:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
        tile = true, tileSize = 16, edgeSize = 16,
        insets = { left = 4, right = 4, top = 4, bottom = 4 }});
    s:SetBackdropColor(0,0,0,1);
    e:HighlightText() -- select all (if to be used for copy paste)
    -- optional/just to close that frame
    e:SetScript("OnEscapePressed", function()
        local txt = e:GetText()

        Import(txt)

        s:Hide()
    end)
end


local function CreateExportFrame()

    local exportText = CreateExportText()
    local s = CreateFrame("ScrollFrame", nil, UIParent, "UIPanelScrollFrameTemplate") -- or you actual parent instead
    s:SetSize(300,200)
    s:SetPoint("CENTER")
    local e = CreateFrame("EditBox", nil, s)
    e:SetMultiLine(true)
    e:SetFontObject(ChatFontNormal)
    e:SetWidth(300)
    s:SetScrollChild(e)
    --- demo multi line text
    e:SetText(exportText)
    s:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
        tile = true, tileSize = 16, edgeSize = 16,
        insets = { left = 4, right = 4, top = 4, bottom = 4 }});
    s:SetBackdropColor(0,0,0,1);
    e:HighlightText() -- select all (if to be used for copy paste)
    -- optional/just to close that frame
    e:SetScript("OnEscapePressed", function()
        local txt = e:GetText()
        s:Hide()
    end)

    e:SetScript("OnTextChanged", function()
        e:SetText(exportText)
        e:HighlightText()
    end)
end


local function GankAddonCommands(msg, editbox, slashname)
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")

    -- first run
    if Gankr == nil then
        Gankr = {}
        Gankr["index"] = 0
    end

    if cmd == "remove" then
        if args ~= "" then
            RemoveNameFromGankTable(args)
        else
            print("Missing name. Try /gank for commands")
        end

    elseif cmd == "export" then
        CreateExportFrame()

    elseif cmd == "import" then

        CreateImportFrame()

    elseif cmd == "reset" then

        Gankr = {}
        Gankr["index"] = 0

    elseif cmd == "list" then

        if Gankr == nil then
            return
        end

        ListGankers()

    elseif cmd == "target" then
        local name = UnitName("target")
        if name ~= nil and name ~= "" then
            AddNameToGanklist(name, args)
        else
            print("Missing target.")
        end
    elseif cmd == "add" and args ~= "" then
        AddNameToGanklist(args)
    elseif cmd ~= nil and cmd ~= "" and args == "" then
        AddNameToGanklist(msg)
    elseif cmd ~= nil and cmd ~= "" and args ~= "" then
        AddNameToGanklist(cmd, args)
    else
        local slash = "/gank"
        if slashname ~= nil then
            slash = slashname
        end

        print("Misaros Revenge Addon. Usage:")
        print("/gank /revenge /rev - All do the same")
        print(slash .. " {name} - Adds the name to the gank table")
        print(slash .. " remove {name} - Removes the name from the gank table")
        print(slash .. " {name} {reason} - Adds the name to the gank tables with the specified reason")
        print(slash .. " target - Adds the current target to the gank table")
        print(slash .. " list - Prints all names in the gank table")
        print(slash .. " import - Creates import window, Paste your import Data here")
        print(slash .. " export - Creates export window, Ready for Copy Paste")
        print(slash .. " reset - Reset all data")
    end
end

local function RevengeAddonCommands(msg, editbox)
    GankAddonCommands(msg, editbox, '/rev')
end

SLASH_GANK1 = '/gank'
SLASH_REVENGE1, SLASH_REVENGE2 = '/rev', '/revenge'
SlashCmdList["GANK"] = GankAddonCommands
SlashCmdList["REVENGE"] = RevengeAddonCommands
